from .bloc import Bloc 
from .disjonction import Disjonction 
from .conjonction import Conjonction

class Circuit:
    """ Modélise un circuit électrique composé de blocs et de connexions. """
    
    def __init__(self):
        self.blocs = {}      
        self.conflits = []
        self.no_goods = []      
        
    def ajouter_bloc(self, nom, entree_1, entree_2, sortie):
        """ Ajoute un bloc au circuit.
        
            :param nom: un nom identifiant le bloc.
            :param entree_1: le nom de l'une des entrées du bloc.
            :param entree_2: le nom de l'autre entrée.
            :param sortie: le nom de la connexion de sortie.
        """
        
        bloc = Bloc(nom, entree_1, entree_2, sortie)
        self.blocs[sortie] = bloc
        
    def observe_incorrect(self, signal):
        """ 
            Notifie l'observation d'un signal incorrect. 
        
            :param signal: le nom de la connection observée.
        """
        
        chemin = self.trouver_un_chemin(signal)
        conflit = self.traduire_chemin_en_disjonction(chemin)
        self.conflits.append(conflit)

    def observe_correct(self, signal):
        """ 
            Notifie l'observation d'un signal correct. 
            
            :param signal: le nom de la connection observée.            
        """
        
        chemin = self.trouver_un_chemin(signal)
        for el in chemin:
            self.no_goods.append(Conjonction([el]))

    def trouver_un_chemin(self, signal):
        """ 
            Collecte la liste des candidats possibles pour expliquer 
            l'observation d'un signal défectueux.
            
            :param signal: le nom de la connection observée.
        
        """
        
        chemin = []
        sorties = [signal]
        while len(sorties) > 0:
            sortie = sorties.pop(-1)
            bloc = self.blocs.get(sortie)
            if bloc is not None:
                chemin.append(bloc)
                sorties.append(bloc.entree_1)
                sorties.append(bloc.entree_2)
        return chemin

    def traduire_chemin_en_disjonction(self, chemin):
        """ 
            Transforme un chemin en une disjonction de conjonctions,
            c'est-à-dire un conflit.
        """
      
        return Disjonction([Conjonction([etape]) for etape in chemin])
